package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Representing square polynomials having coefficients of type <code>int</code>.</p>
 *
 * <p>A square polynomial \( p(x)  = a x² + b x + c \) is being defined by its three coefficients
 * \(a\), \(b\) and \(c\). We assume all three to be of type <code>int</code>.</p>
 *
 *
 * <h2>Calculating polynomial values.</h2>
 *
 *   <table  class="goikTableDefaults">
 *       <caption>Sample code calculating polynomial values</caption>
 *       <tbody>
 *           <tr>
 *               <th>Code</th>
 *               <th>Result</th>
 *           </tr>
 *           <tr>
 *               <td>
 *                   <pre><code class="language-java"> final SquarePolynom poly =   // Representing
 * new SquarePolynom(-1, 2, 1); // p(x) = -x² + 2x + 1
 *
 * double y = poly.get(3.0); // -1 * 3.0 * 3.0 + 2 * 3.0 + 1
 *
 * System.out.println(y);</code></pre>
 *       </td>
 *       <td>
 *          <pre><code class="nohighlight">-2</code></pre>
 *       </td>
 *     </tr>
 *   </tbody>
 * </table>
 *
 * <h2>Providing zeroes (German: "Nullstellen") of square polynomials.</h2>
 *
 * <p>Zeroes are being calculated using:</p>
 *
 * \[ x_{1,2} = {{-b \pm \sqrt{b^2 - 4 ac}} \over {2a}} \]
 *
 * <p>There are two main cases \( \mathbf{a \neq 0}\) and \( \mathbf{a = 0}\):</p>
 *
 * <h3>Case \( \mathbf{a \neq 0}\), true square polynomial</h3>
 *
 * <p>There are three sub cases:</p>
 *
 * <dl>
 *     <dt>\(\mathbf{ 0 &lt; b^2 - 4 ac}\):</dt>
 *     <dd>
 *         <p>Two zeroes \(x_1\) and \(x_2\) according to the above formular:</p>
 *
 *         <table  class="goikTableDefaults">
 *             <caption>Sample code illustrating zero values calculation</caption>
 *             <tbody>
 *                 <tr>
 *                     <th>Code</th>
 *                     <th>Result</th>
 *                 </tr>
 *                 <tr>
 *                     <td>
 *                         <pre><code class="language-java"> final SquarePolynom poly =         // Representing
 *     new SquarePolynom(4, -3, -10); // p(x) = 4x² - 3x - 10
 *
 * double[] zeroes = poly.getZeroes();
 *
 * System.out.println("Found " + zeroes.length + " zeroes:");
 *
 * System.out.println("x_1 = " + zeroes[0]);
 * System.out.println("x_2 = " + zeroes[1]);</code></pre>
 *             </td>
 *                     <td>
 *                         <pre><code class="nohighlight"> Found 2 zeroes:
 * x_1 = -1.25
 * x_2 = 2.0</code></pre>
 *                     </td>
 *                 </tr>
 *             </tbody>
 *         </table>
 *
 *         <p>Note the two values in the array being ordered by value like <code>double[] zeroes = {-1.25, 2.0}</code>
 *           rather than <code>... {2.0, -1.25}</code></p>
 *     </dd>
 *
 *     <dt>\(\mathbf{  0 = b^2 - 4 ac}\):</dt>
 *
 *     <dd>
 *         <p>One zero \(x = {-b\over 2a}\).</p>
 *
 *         <p>We allow for changing a polynomial's coefficients. Continuing from the above example we have:</p>
 *
 *         <table class="goikTableDefaults">
 *             <caption>Changing coefficients</caption>
 *             <tbody>
 *                 <tr>
 *                     <th>Code</th>
 *                     <th>Result</th>
 *                 </tr>
 *                 <tr>
 *                     <td>
 *                         <pre><code class="java"> ...
 *         poly.setA(1);  // Representing
 *         poly.setB(-8); // p(x) = x² -8x + 16
 *         poly.setC(16);
 *
 *         zeroes = poly.getZeroes();
 *         System.out.println("Found " + zeroes.length + " zero:");
 *
 *         System.out.println("x_1 = " + zeroes[0]);</code></pre>
 *                     </td>
 *                     <td>
 *                         <pre><code class="nohighlight"> ...
 *         Found 1 zero:
 *         x_1 = 4.0</code></pre>
 *                     </td>
 *                 </tr>
 *             </tbody>
 *         </table>
 *         <p>&nbsp;</p>
 *     </dd>
 *
 *     <dt>\(\mathbf{  b^2 - 4 ac &lt; 0 }\):</dt>
 *
 *     <dd>
 *         <p>Square root of negative thus no real valued solution:</p>
 *
 *         <table class="goikTableDefaults">
 *             <caption>Changing coefficients again</caption>
 *             <tbody>
 *                 <tr>
 *                     <th>Code</th>
 *                     <th>Result</th>
 *                 </tr>
 *                 <tr>
 *                     <td>
 *                         <pre><code class="java"> ...
 *         poly.setA(1);  // Representing
 *         poly.setB(0);  // p(x) = x² + 1
 *         poly.setC(1);  // (No real valued zero)
 *
 *         zeroes = poly.getZeroes();
 *         System.out.println("Found " + zeroes.length + " zero");</code></pre>
 *                     </td>
 *                     <td>
 *                         <pre><code class="nohighlight"> ...
 *         Found 0 zero</code></pre>
 *                     </td>
 *                 </tr>
 *             </tbody>
 *         </table>
 *     </dd>
 * </dl>
 *
 * <h3>Case \( \mathbf{a = 0}\), linear polynomial</h3>
 *
 * <p>This leaves us with just a linear polynomial \(bx + c\) also resulting in three different sub cases:</p>
 *
 *
 * <dl>
 *     <dt>\( \mathbf{b \neq 0} :\)</dt>
 *
 *     <dd>
 *          <p>Exactly one zero</p>
 *     </dd>
 *
 *     <dt>\( \mathbf{b = 0 \ \text{and} \ c \neq 0 :}\)</dt>
 *     <dd>
 *         <p>No zero at all</p>
 *     </dd>
 *
 *     <dt>\(\mathbf{b = 0  \ \text{and} \ c = 0} \):</dt>
 *     <dd>
 *         <p>An infinite number of zeroes. Since this cannot be represented by a finite <code>double[]</code> array
 *         an {@link ArithmeticException} is being thrown like in the subsequent code sample:</p>
 *
 *         <table class="goikTableDefaults">
 *             <caption>Dealing with \( \mathbf{a = b = c = 0} \)</caption>
 *             <tbody>
 *                <tr>
 *                  <th>Code</th>
 *                  <th>Result</th>
 *                </tr>
 *                <tr>
 *                  <td>
 *                   <pre><code class="java"> ...
 * poly.setA(0);
 * poly.setB(0);
 * poly.setC(0);
 * zeroes = poly.getZeroes();</code></pre>
 *                  </td>
 *                  <td>
 *                    <pre><code class="nohighlight"> ...
 * Exception in thread "main" java.lang.ArithmeticException:
 *     <b style="color:red;">Polynomial is identical to zero function</b></code></pre>
 *                  </td>
 *                </tr>
 *              </tbody>
 *         </table>
 *
 *     </dd>
 * </dl>
 *
 * <section class="implementationHints">
 *    <h2>Hints:</h2>
 *
 *   <ul>
 *
 *     <li>This class is yet unimplemented. The above code snippets provide a clue to an implementation
 *        satisfying the corresponding unit tests.</li>
 *
 *     <li><a href=
 *       "https://freedocs.mi.hdm-stuttgart.de/lib/openjdk-21-doc/api/java.base/java/lang/Math.html#sqrt(double)"
 *       >Math.sqrt(...)</a>
 *     </li>
 *
 *   </ul>
 *
 * </section>
 */
public class SquarePolynom {
    // Implement me
}
