package de.hdm_stuttgart.mi.sd1.ignore_me;

import org.junit.Assert;

import java.lang.reflect.*;
import java.util.Arrays;
import java.util.StringJoiner;
import java.util.stream.Collectors;

public class ObjectWrapper<T> {

  private final Class classz;
  private final T instance;

  static public void assertFinal(final Class classZ, final String fieldName) {
    try {
      final int modifier = classZ.getDeclaredField(fieldName).getModifiers();
      Assert.assertTrue("»" + fieldName + "« can be modified, no immutable!", Modifier.isFinal(modifier));

    } catch (NoSuchFieldException e) {
      Assert.fail("No such field »" + fieldName + "«");
    }
  }

  public ObjectWrapper(final Class<T> classz, final Object ... parameter) {
    this.classz = classz;
    T tmpInstance = null;
    final Constructor<?>[] candidates = Arrays.
            stream(classz.getDeclaredConstructors()).
            filter(c -> matchesArgumentList(c, parameter)).
            toArray(Constructor<?>[]::new);
    if (0 == candidates.length) {
      Assert.fail("No suitable constructor in class »" + classz.getName() + "« matching arguments »(" +
              Arrays.stream(parameter).map(Object::toString).collect(Collectors.joining(", ")) +
              ")«");
    } else if (1 < candidates.length) {
      Assert.fail("Multiple constructor matches due to ambiguous signature");
    } else {
      try {
        tmpInstance = (T) candidates[0].newInstance(
                Arrays.stream(parameter).toArray());
      } catch (final IllegalAccessException e) {
        Assert.fail("Unable to instantiate instance of class »" + classz.getName() + "«:\n" +
                "Cannot access constructor " + candidates[0].toString());
      } catch (Exception e) {
        Assert.fail("Unable to instantiate instance of class »" + classz.getName() + "«:\n  "+ e);
      }
    }
    instance = tmpInstance;
  }

  public ObjectWrapper(final Class<T> classz, final Class<? extends Throwable> expectedException, final Object ... parameter) {
    this.classz = classz;
    T tmpInstance = null;
    final Constructor<?>[] candidates = Arrays.
            stream(classz.getConstructors()).
            filter(c -> matchesArgumentList(c, parameter)).
            toArray(Constructor<?>[]::new);
    if (0 == candidates.length) {
      Assert.fail("No suitable constructor in class »" + classz.getName() + "« matching arguments »(" +
              Arrays.stream(parameter).map(Object::toString).collect(Collectors.joining(", ")) +
              ")«");
    } else if (1 < candidates.length) {
      Assert.fail("Multiple constructor matches due to ambiguous signature");
    } else {
      try {
        tmpInstance = (T) candidates[0].newInstance(
                Arrays.stream(parameter).toArray());
        Assert.fail("Expected exception of type »" + expectedException.getName() + "« to be thrown");
      } catch (InstantiationException|IllegalAccessException|IllegalArgumentException|InvocationTargetException e) {
        if (e.getCause().getClass() != expectedException) {
          Assert.fail("Unable to instantiate: " + e + ", cause:\n  Expected exception of type »" +
                  expectedException.getName() + "« but was »"+ e.getCause().getClass().getName() + "«");
        }
      }
    }
    instance = tmpInstance;
  }

  public <R> void assertFieldExists(final Class<R> valueType, final String name) {
    try {
      final Field field = classz.getField(name);
      if (!valueType.equals(field.getType())) {
        Assert.fail("Field »" + name + "« in class »" + classz.getName() +  "« is not of type »" +
                valueType.getName() + "«");
      }
    } catch (final NoSuchFieldException e) {
      Assert.fail("No such field »" + name + "« in class »" + classz.getName() + "«");
    }
  }

  public <R> R get(final Class<R> valueType, final String name) {

    try {
      final Field field = classz.getField(name);
      if (valueType.equals(field.getType())) {
        return (R) field.get(instance);
      } else {
        Assert.fail("Field »" + name + "« in class »" + classz.getName() +  "« is not of type »" +
                valueType.getName() + "«");
      }
    } catch (final NoSuchFieldException e) {
      Assert.fail("No such field »" + name + "« in class »" + classz.getName() + "«");
    } catch (final IllegalAccessException e) {
      Assert.fail("Unable to access field »" + name + "« in class »" + classz.getName() + "«: " + e.getMessage());
    }
    return null;
  }
  public void set(final String name, final Object value) {

    try {
      final Field field = classz.getField(name);
      final Class  argumentType = value.getClass();
      if (field.getType().equals(argumentType) ||
              field.getType().equals(getPrimitiveType(argumentType))){
       field.set(instance, value);
      } else {
        Assert.fail("Field »" + name + "« in class »" + classz.getName() +  "« is not of type »" +
                argumentType.getName() + "«");
      }
    } catch (final NoSuchFieldException e) {
      Assert.fail("No such field »" + name + "« in class »" + classz.getName() + "«");
    } catch (final IllegalAccessException e) {
      Assert.fail("Unable to access field »" + name + "« in class »" + classz.getName() + "«: " + e.getMessage());
    }
  }

  public <R> R invoke(final Class<R> returnType, final String name, final Object ... parameter) {

    final Method[] candidates = Arrays.
            stream(classz.getDeclaredMethods()).
            filter(m-> m.getName().equals(name) && matchesArgumentList(m, parameter)).
            toArray(Method[]::new);

    if (0 == candidates.length) {
      Assert.fail("No suitable method "  + getSignature(returnType, name, parameter) +
              " found");
    } else if (1 < candidates.length) {
      Assert.fail("Multiple method matches due to ambiguous signature");
    } else {
      final Method method = candidates[0];
      if (method.getReturnType().equals(returnType)) {
        try {
          return (R) method.invoke(instance, parameter);
        } catch (final IllegalAccessException e) {
          Assert.fail("Unable to execute existing method: »" + method.toString() + "«");
        }catch (final IllegalArgumentException|InvocationTargetException e) {
          Assert.fail("Unable to execute existing method: »" + method.toString() + "«\n" +
                  e + ", cause:" + e.getCause());
        }
      } else {
        Assert.fail("Method »" + method.getName() + "« does have return type »"  + method.getReturnType() + "«" +
                "rather then »" + returnType.getName() + "«");
      }
    }
    return null;
  }

  private <R> StringBuilder getSignature(final Class<R> returnType, final String name, final Object ... parameter) {

    final StringBuilder ret = new StringBuilder();
    appendType(returnType, ret);

    return ret.append(' ').append(name).append('(')
            .append(Arrays.stream(parameter).map(this::appendObjectsType).collect(Collectors.joining(", ")))
            .append(')');
  }

  private StringBuilder appendObjectsType (final Object object) {
    final StringBuilder builder = new StringBuilder();
    if (null == object) {
        builder.append("null");
      } else {
        appendType(object.getClass(), builder);
      }
    return builder;
  }

  private <R> void appendType(final Class<R> type, StringBuilder result) {

    if (type.isArray()) {
      final Class <?> componentType = type.getComponentType();
      result.append(componentType.getName()).append("[]");
    } else {
      result.append(type.getName());
    }
  }

  public <R> R invoke(final Class<R> returnType, final String name,
                                       final Class<? extends Throwable> expectedException, final Object ... parameter) {

    final Method[] candidates = Arrays.
            stream(classz.getMethods()).
            filter(m-> m.getName().equals(name) && matchesArgumentList(m, parameter)).
            toArray(Method[]::new);

    if (0 == candidates.length) {
      Assert.fail("No suitable method found");
    } else if (1 < candidates.length) {
      Assert.fail("Multiple method matches due to ambiguous signature");
    } else {
      final Method method = candidates[0];
      if (method.getReturnType().equals(returnType)) {
        try {
          R ret = (R) method.invoke(instance, parameter);
          Assert.fail("Expected exception of type »" + expectedException.getName() + "«");
          return ret;
        } catch (final IllegalAccessException| IllegalArgumentException|InvocationTargetException e) {
          if (e.getCause().getClass() != expectedException) {
            Assert.fail("Unable to execute method: " + e + ", cause:\n  Expected exception of type »" +
                    expectedException.getName() + "« but was »"+ e.getCause().getClass().getName() + "«");
          }
        }
      } else {
        Assert.fail("Method »" + method.getName() + "« does have return type »"  + method.getReturnType() + "«" +
                "rather then »" + returnType.getName() + "«");
      }
    }
    return null;
  }

  /**
   * Check for a given array of objects matching an {@link Executable}'s argument list.
   *
   * @param executable
   * @param parameter
   * @return <code>true</code> if parameters match the {@link Executable}'s argument list, <code>false</code>
   * otherwise.
   */
  private boolean matchesArgumentList(final Executable executable, final Object ... parameter) {

    if (executable.getParameterCount() != parameter.length) {
      return false;
    } else {
      final Class<?>[] formalArgumentTypes = executable.getParameterTypes();
      for (int i = 0; i < formalArgumentTypes.length; i++) {
        final Class parametersClass = parameter[i].getClass();

        if (!formalArgumentTypes[i].equals(parametersClass) &&
                !formalArgumentTypes[i].equals(getPrimitiveType(parametersClass))) {
          return false;
        }
      }
    }
    return true;
  }

  static private Class<?> getPrimitiveType (final Class<?> in) {
    if (in.equals(Byte.class)) {
      return byte.class;
    } else if (in.equals(Short.class)) {
      return short.class;
    } else if (in.equals(Integer.class)) {
      return int.class;
    } else if (in.equals(Long.class)) {
      return long.class;
    } else if (in.equals(Float.class)) {
      return float.class;
    } else if (in.equals(Double.class)) {
      return double.class;
    } else if (in.equals(Boolean.class)) {
      return boolean.class;
    } else if (in.equals(Character.class)) {
      return char.class;
    } else {
      return in;            // Type is no primitive
    }
  }

}
